@@ -58,8 +58,8 @@
 #*/###########################################################################
 setMethodS3("callSegmentationOutliers", "default", function(y, chromosome=0, x=NULL, method="DNAcopy::smooth.CNA", ..., verbose=FALSE) {
   # Local copies of DNAcopy functions
-  CNA <- .useDNAcopy("CNA");
-  smooth.CNA <- .useDNAcopy("smooth.CNA");
+  CNA <- .use("CNA", package="DNAcopy");
+  smooth.CNA <- .use("smooth.CNA", package="DNAcopy");
 
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
