@@ -32,7 +32,7 @@
 #*/###########################################################################
 setMethodS3("findNeutralCopyNumberState", "default", function(C, isAI, weights=NULL, ..., minDensity=1e-10, flavor=c("firstPeak", "maxPeak"), verbose=FALSE) {
   # This will load the 'aroma.light' namespace, if not already done.
-  findPeaksAndValleys <- .useAromaLight("findPeaksAndValleys");
+  findPeaksAndValleys <- .use("findPeaksAndValleys", package="aroma.light");
 
 
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
